#!/usr/bin/env python
#
# Build the LEX/YACC table modules for the MOF compiler, in the PyWBEM
# installation directory. The installation directory is used by removing
# the directory of this script from the Python module search path.

import sys
import os
from pprint import pprint

myname = os.path.basename(sys.argv[0])

DEBUG = False   # Enable to get debug info
VERBOSE = True  # TODO: Pass as command line option


def env_info():
    print("%s: Platform: %s" % (myname, sys.platform))
    print("%s: Python executable: %s" % (myname, sys.executable))
    print("%s: Python version: %s" % (myname, sys.version))
    print("%s: Python module search path:" % myname)
    pprint(sys.path)


def main():

    if VERBOSE:
        print("%s: Rebuilding the pywbem LEX/YACC table modules, if needed" % \
              myname)

    # We want to build the table modules in the target install directory.
    #
    # The mof_compiler._build() function builds them in the directory where
    # the mof_compiler module was imported from.
    #
    # This script runs in the directory where the distribution archive has
    # been unpacked to (or during development in the git repo work directory),
    # where the pywbem package is directly available as a subdirectory.
    #
    # Therefore, we need to remove the directory of this script from the Python
    # module search path before importing the mof_compiler module, in order to
    # import it from the target install directory.
    if VERBOSE:
        print("%s: Ensuring to build in pywbem install directory" % myname)
    my_abs_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    removals = []
    for i, p in enumerate(sys.path):
        abs_p = os.path.abspath(p)
        if abs_p == my_abs_path:
            if DEBUG:
                print("%s: Debug: Removing from module search path: %s" % \
                      (myname, abs_p))
            removals.append(i)
    for i in reversed(removals):
        del sys.path[i]
    if DEBUG:
        print("%s: Debug: Resulting module search path:" % myname)
        pprint(sys.path)

    try:
        import pywbem
        if DEBUG:
            print("%s: Debug: Pywbem package successfully imported from: %s" % \
                  (myname, pywbem.__file__))
    except ImportError as exc:
        print("%s: Error: Import of pywbem package failed: %s" % (myname, exc))
        env_info()
        return 1

    from pywbem import mof_compiler
    try:
        mof_compiler._build(verbose=True)
    except Exception as exc:
        print("%s: Error: Rebuilding the pywbem LEX/YACC table modules " \
              "failed: %s" % (myname, exc))
        env_info()
        return 1

    if VERBOSE:
        print("%s: Successfully rebuilt the pywbem LEX/YACC table modules, "
              "if needed" % myname)
    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/bin/env python
#
# (C) Copyright 2006,2007 Hewlett-Packard Development Company, L.P.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author: Tim Potter <tpot@hp.com>
# Author: Ross Peoples <ross.peoples@gmail.com>
#

"""CIM/XML Parser. Parses the CIM/XML Elements defined in the DMTF
   specification DSP0201.

   WARNING: Many of the parsing functions defined in this file must
   keep the exact name (i.e. those prepended by parse_) since the
   parser uses the names to call the function from the parse_any(...)
   function substituting _ for the "." in the element names.

   These functions raise ParseError exception if there are any
   errors in the parsing and terminate the parsing.
"""
from __future__ import print_function
from xml.dom import pulldom
import sys
import six

from .cim_obj import CIMInstance, CIMInstanceName, CIMQualifier, \
                     CIMProperty, tocimobj
from .tupleparse import ParseError

__all__ = []

#class ParseError(Exception):
#    """This exception is raised when there is a validation error detected
#    by the parser."""
#    pass

#
# Helper functions
#

def _get_required_attribute(node, attr):
    """Return an attribute by name.  Raise ParseError if not present."""

    if not node.hasAttribute(attr):
        raise ParseError(
            'Expecting %s attribute in element %s' % (attr, node.tagName))

    return node.getAttribute(attr)

def _get_attribute(node, attr):
    """Return an attribute by name, or None if not present."""

    if node.hasAttribute(attr):
        return node.getAttribute(attr)

    return None

def _get_end_event(parser, tagName):
    """Check that the next event is the end of a particular XML tag."""

    (event, node) = six.next(parser)

    if event != pulldom.END_ELEMENT or node.tagName != tagName:
        raise ParseError(
            'Expecting %s end tag, got %s %s' % (tagName, event, node.tagName))

def _is_start(event, node, tagName):  # pylint: disable=invalid-name
    """Return true if (event, node) is a start event for tagname."""

    return event == pulldom.START_ELEMENT and node.tagName == tagName

def _is_end(event, node, tagName): # pylint: disable=invalid-name
    """Return true if (event, node) is an end event for tagname."""

    return event == pulldom.END_ELEMENT and node.tagName == tagName

# <!-- ************************************************** -->
# <!-- Root element                                       -->
# <!-- ************************************************** -->

# <!ELEMENT CIM (MESSAGE | DECLARATION)>
# <!ATTLIST CIM
#       CIMVERSION CDATA #REQUIRED
#       DTDVERSION CDATA #REQUIRED
# >

# <!-- ************************************************** -->
# <!-- Object declaration elements                        -->
# <!-- ************************************************** -->

# <!ELEMENT DECLARATION (DECLGROUP | DECLGROUP.WITHNAME | DECLGROUP.WITHPATH)+>
# <!ELEMENT DECLGROUP ((LOCALNAMESPACEPATH | NAMESPACEPATH)?,
#                      QUALIFIER.DECLARATION*, VALUE.OBJECT*)>
# <!ELEMENT DECLGROUP.WITHNAME ((LOCALNAMESPACEPATH | NAMESPACEPATH)?,
#                               QUALIFIER.DECLARATION*, VALUE.NAMEDOBJECT*)>
# <!ELEMENT DECLGROUP.WITHPATH (VALUE.OBJECTWITHPATH |
#                               VALUE.OBJECTWITHLOCALPATH)*>
# <!ELEMENT QUALIFIER.DECLARATION (SCOPE?, (VALUE | VALUE.ARRAY)?)>
# <!ATTLIST QUALIFIER.DECLARATION
#       %CIMName;
#       %CIMType;                #REQUIRED
#       ISARRAY     (true|false) #IMPLIED
#       %ArraySize;
#       %QualifierFlavor;
# >
# <!ELEMENT SCOPE EMPTY>
# <!ATTLIST SCOPE
#       CLASS       (true | false) "false"
#       ASSOCIATION (true | false) "false"
#       REFERENCE   (true | false) "false"
#       PROPERTY    (true | false) "false"
#       METHOD      (true | false) "false"
#       PARAMETER   (true | false) "false"
#       INDICATION  (true | false) "false"
# >

# <!-- ************************************************** -->
# <!-- Object Value elements                              -->
# <!-- ************************************************** -->

# <!ELEMENT VALUE (#PCDATA)>

def parse_value(parser, event, node): # pylint: disable=unused-argument
    """ Parse CIM/XML VALUE element and return the value"""
    value = ''

    (next_event, next_node) = six.next(parser)

    if next_event == pulldom.CHARACTERS:

        value = next_node.nodeValue

        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'VALUE'):
        raise ParseError('Expecting end VALUE')

    return value

# <!ELEMENT VALUE.ARRAY (VALUE*)>

def parse_value_array(parser, event, node): #pylint: disable=unused-argument
    # pylint: disable=invalid-name
    """ Parse CIM/XML VALUE.ARRAY element and return the value array"""

    value_array = []

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'VALUE'):

        value_array.append(parse_value(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_end(next_event, next_node, 'VALUE.ARRAY'):
                break

            if _is_start(next_event, next_node, 'VALUE'):
                value_array.append(parse_value(parser, next_event, next_node))
            else:
                raise ParseError('Expecting VALUE element')

    return value_array

# <!ELEMENT VALUE.REFERENCE (CLASSPATH | LOCALCLASSPATH | CLASSNAME |
#                            INSTANCEPATH | LOCALINSTANCEPATH |
#                            INSTANCENAME)>

def parse_value_reference(parser, event, node): #pylint: disable=unused-argument

    """Parse CIM/XML VALUE.REFERENCE element and call function
       for the child ELEMENT. Return the result of that element
       parser call.
    """

    (next_event, next_node) = six.next(parser)

    # TODO 2/16 KS: Functions below do not exist: i.e. class stuff broken:
    #               parse_classpath, parse_localclasspath, parse_classname.

    if _is_start(next_event, next_node, 'CLASSPATH'):
        result = parse_classpath(parser, next_event, next_node)

    elif _is_start(next_event, next_node, 'LOCALCLASSPATH'):
        result = parse_localclasspath(parser, next_event, next_node)

    elif _is_start(next_event, next_node, 'CLASSNAME'):
        result = parse_classname(parser, next_event, next_node)

    elif _is_start(next_event, next_node, 'INSTANCEPATH'):
        result = parse_instancepath(parser, next_event, next_node)

    elif _is_start(next_event, next_node, 'LOCALINSTANCEPATH'):
        result = parse_localinstancepath(parser, next_event, next_node)

    elif _is_start(next_event, next_node, 'INSTANCENAME'):
        result = parse_instancename(parser, next_event, next_node)

    else:
        raise ParseError('Expecting (CLASSPATH | LOCALCLASSPATH | CLASSNAME '
                         '| INSTANCEPATH | LOCALINSTANCEPATH | INSTANCENAME)')

    _get_end_event(parser, 'VALUE.REFERENCE')

    return result

# <!ELEMENT VALUE.REFARRAY (VALUE.REFERENCE*)>
# <!ELEMENT VALUE.OBJECT (CLASS | INSTANCE)>
# <!ELEMENT VALUE.NAMEDINSTANCE (INSTANCENAME, INSTANCE)>
# <!ELEMENT VALUE.NAMEDOBJECT (CLASS | (INSTANCENAME, INSTANCE))>
# <!ELEMENT VALUE.OBJECTWITHLOCALPATH ((LOCALCLASSPATH, CLASS) |
#                                      (LOCALINSTANCEPATH, INSTANCE))>
# <!ELEMENT VALUE.OBJECTWITHPATH ((CLASSPATH, CLASS) |
#                                 (INSTANCEPATH, INSTANCE))>
# <!ELEMENT VALUE.NULL EMPTY>

# <!-- ************************************************** -->
# <!-- Object naming and locating elements                -->
# <!-- ************************************************** -->

def parse_namespacepath(parser, event, node): #pylint: disable=unused-argument
    """Parse namespace path element and return tuple of
       host and namespace
           <!ELEMENT NAMESPACEPATH (HOST, LOCALNAMESPACEPATH)>
    """


    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'HOST'):
        raise ParseError('Expecting HOST')

    host = parse_host(parser, next_event, next_node)

    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'LOCALNAMESPACEPATH'):
        raise ParseError('Expecting LOCALNAMESPACEPATH')

    namespacepath = parse_localnamespacepath(parser, next_event, next_node)

    (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'NAMESPACEPATH'):
        raise ParseError('Expecting end NAMESPACEPATH')

    return (host, namespacepath)


def parse_localnamespacepath(parser, event, node):
    #pylint: disable=unused-argument
    """Parse LOCALNAMESPACEPATH for Namespace. Return assembled namespace
            <!ELEMENT LOCALNAMESPACEPATH (NAMESPACE+)>
    """

    (next_event, next_node) = six.next(parser)

    namespaces = []

    if not _is_start(next_event, next_node, 'NAMESPACE'):
        print(next_event, next_node)
        raise ParseError('Expecting NAMESPACE')

    namespaces.append(parse_namespace(parser, next_event, next_node))

    while 1:

        (next_event, next_node) = six.next(parser)

        if _is_end(next_event, next_node, 'LOCALNAMESPACEPATH'):
            break

        if _is_start(next_event, next_node, 'NAMESPACE'):
            namespaces.append(parse_namespace(parser, next_event, next_node))
        else:
            raise ParseError('Expecting NAMESPACE')

    return '/'.join(namespaces)


def parse_host(parser, event, node):
    """Parse and return the host entity if that is the next entity
           <!ELEMENT HOST (#PCDATA)>
    """
    #pylint: disable=unused-argument

    host = ''

    (next_event, next_node) = six.next(parser)

    if next_event == pulldom.CHARACTERS:

        host = next_node.nodeValue

        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'HOST'):
        raise ParseError('Expecting end HOST')

    return host


def parse_namespace(parser, event, node):
    #pylint: disable=unused-argument
    """Parse the CIM/XML NAMESPACE element and return the value
       of the CIMName attribute
           <!ELEMENT NAMESPACE EMPTY>
           <!ATTLIST NAMESPACE
                 %CIMName;
    """
    name = _get_required_attribute(node, 'NAME')

    (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'NAMESPACE'):
        raise ParseError('Expecting end NAMESPACE')

    return name

# <!ELEMENT CLASSPATH (NAMESPACEPATH, CLASSNAME)>

# <!ELEMENT LOCALCLASSPATH (LOCALNAMESPACEPATH, CLASSNAME)>

# <!ELEMENT CLASSNAME EMPTY>
# <!ATTLIST CLASSNAME
#       %CIMName;
# >

# TODO 3/16 KS: MUST implement CLASSPATH, etc.

def parse_instancepath(parser, event, node):
    #pylint: disable=unused-argument
    """Parse the CIM/XML INSTANCEPATH element and return an
       instancname

       <!ELEMENT INSTANCEPATH (NAMESPACEPATH, INSTANCENAME)>
    """

    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'NAMESPACEPATH'):
        raise ParseError('Expecting NAMESPACEPATH')

    host, namespacepath = parse_namespacepath(parser, next_event, next_node)

    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'INSTANCENAME'):
        print(next_event, next_node)
        raise ParseError('Expecting INSTANCENAME')

    instancename = parse_instancename(parser, next_event, next_node)

    instancename.host = host
    instancename.namespace = namespacepath

    return instancename


def parse_localinstancepath(parser, event, node):
    """Parse LOCALINSTANCEPATH element returning instancename
           <!ELEMENT LOCALINSTANCEPATH (LOCALNAMESPACEPATH, INSTANCENAME)>
    """

    #pylint: disable=unused-argument

    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'LOCALNAMESPACEPATH'):
        raise ParseError('Expecting LOCALNAMESPACEPATH')

    namespacepath = parse_localnamespacepath(parser, next_event, next_node)

    (next_event, next_node) = six.next(parser)

    if not _is_start(next_event, next_node, 'INSTANCENAME'):
        raise ParseError('Expecting INSTANCENAME')

    instancename = parse_instancename(parser, next_event, next_node)

    instancename.namespace = namespacepath

    return instancename


def parse_instancename(parser, event, node): #pylint: disable=unused-argument
    """Parse INSTANCENAME element returning CIMInstancename containing
       classname and keybindings
            <!ELEMENT INSTANCENAME (KEYBINDING* | KEYVALUE? | VALUE.REFERENCE?)>
            <!ATTLIST INSTANCENAME
                  %ClassName;
            >
    """

    classname = _get_required_attribute(node, 'CLASSNAME')
    keybindings = []

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'KEYBINDING'):

        keybindings.append(parse_keybinding(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_end(next_event, next_node, 'INSTANCENAME'):
                break

            if _is_start(next_event, next_node, 'KEYBINDING'):
                keybindings.append(
                    parse_keybinding(parser, next_event, next_node))
            else:
                raise ParseError('Expecting KEYBINDING element')

    if _is_end(next_event, next_node, 'INSTANCENAME'):
        pass

    elif _is_start(next_event, next_node, 'KEYVALUE'):
        keybindings.append(('', parse_keyvalue(parser, next_event, next_node)))

    elif _is_start(next_event, next_node, 'VALUE.REFERENCE'):
        keybindings.append(
            parse_value_reference(parser, next_event, next_node))

    else:
        raise ParseError(
            'Expecting KEYBINDING* | KEYVALUE? | VALUE.REFERENCE')

    return CIMInstanceName(classname, keybindings)

# <!ELEMENT OBJECTPATH (INSTANCEPATH | CLASSPATH)>

# <!ELEMENT KEYBINDING (KEYVALUE | VALUE.REFERENCE)>
# <!ATTLIST KEYBINDING
#       %CIMName;
# >

def parse_keybinding(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML KEYBINDING element returning name, value tuple"""

    name = _get_required_attribute(node, 'NAME')

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'KEYVALUE'):
        keyvalue = parse_keyvalue(parser, next_event, next_node)
        result = (name, keyvalue)

    elif _is_start(next_event, next_node, 'VALUE.REFERENCE'):
        value_reference = parse_value_reference(parser,
                                                next_event, next_node)
        result = (name, value_reference)

    else:
        raise ParseError('Expecting KEYVALUE or VALUE.REFERENCE element')

    _get_end_event(parser, 'KEYBINDING')

    return result

# <!ELEMENT KEYVALUE (#PCDATA)>
# <!ATTLIST KEYVALUE
#       VALUETYPE (string | boolean | numeric) "string"
#       %CIMType;              #IMPLIED
# >

def parse_keyvalue(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/CML KEYVALUE element and return key value based on
       VALUETYPE  or TYPE (future) information
    """

    valuetype = _get_required_attribute(node, 'VALUETYPE')

    # TODO 2/16 KS: Type attribute not used. Extend to use. Type  was late
    #               extension to spec to allow real types.
    cim_type = _get_attribute(node, 'TYPE')  # pylint: disable=unused-variable

    (next_event, next_node) = six.next(parser)

    if next_event != pulldom.CHARACTERS:
        raise ParseError('Expecting character data')

    value = next_node.nodeValue

    if valuetype == 'string':
        pass

    elif valuetype == 'boolean':

        # CIM-XML says "These values MUST be treated as
        # case-insensitive" (even though the XML definition
        # requires them to be lowercase.)

        p = value.strip().lower()

        if p == 'true':
            value = True
        elif p == 'false':
            value = False
        else:
            raise ParseError('invalid boolean value %r' % p)

    elif valuetype == 'numeric':

        try:

            # XXX: Use TYPE attribute to create named CIM type.
            # if 'TYPE' in attrs(tt):
            #    return tocimobj(attrs(tt)['TYPE'], p.strip())

            # XXX: Would like to use long() here, but that tends to cause
            # trouble when it's written back out as '2L'
            # pylint: disable=redefined-variable-type
            # Redefined from bool to int
            # pylint: disable=redefined-variable-type
            value = int(value.strip(), 0)

        except ValueError:
            raise ParseError(
                'invalid numeric value "%s"' % value)

    else:
        raise ParseError('Invalid VALUETYPE')

    _get_end_event(parser, 'KEYVALUE')

    return value

# <!-- ************************************************** -->
# <!-- Object definition elements                         -->
# <!-- ************************************************** -->

# <!ELEMENT CLASS (QUALIFIER*, (PROPERTY | PROPERTY.ARRAY |
#                  PROPERTY.REFERENCE)*, METHOD*)>
# <!ATTLIST CLASS
#       %CIMName;
#       %SuperClass;
# >

# <!ELEMENT INSTANCE (QUALIFIER*, (PROPERTY | PROPERTY.ARRAY |
#                     PROPERTY.REFERENCE)*)>
# <!ATTLIST INSTANCE
#       %ClassName;
#       xml:lang   NMTOKEN      #IMPLIED
# >

def parse_instance(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML INSTANCE Element and return CIMInstance"""

    classname = _get_required_attribute(node, 'CLASSNAME')

    properties = []
    qualifiers = []

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'QUALIFIER'):

        qualifiers.append(parse_qualifier(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_start(next_event, next_node, 'PROPERTY') or \
               _is_start(next_event, next_node, 'PROPERTY.ARRAY') or \
               _is_start(next_event, next_node, 'PROPERTY.REFERENCE') or \
               _is_end(next_event, next_node, 'INSTANCE'):
                break

            if _is_start(next_event, next_node, 'QUALIFIER'):
                qualifiers.append(
                    parse_qualifier(parser, next_event, next_node))
            else:
                raise ParseError('Expecting QUALIFIER')

    while 1:

        if _is_end(next_event, next_node, 'INSTANCE'):
            break

        if _is_start(next_event, next_node, 'PROPERTY'):
            properties.append(parse_property(parser, next_event, next_node))

        elif _is_start(next_event, next_node, 'PROPERTY.ARRAY'):
            properties.append(
                parse_property_array(parser, next_event, next_node))

        elif _is_start(next_event, next_node, 'PROPERTY.REFERENCE'):
            properties.append(
                parse_property_reference(parser, next_event, next_node))
        else:
            raise ParseError(
                'Expecting (PROPERTY | PROPERTY.ARRAY | PROPERTY.REFERENCE)')

        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'INSTANCE'):
        raise ParseError('Expecting end INSTANCE')

    return CIMInstance(
        classname,
        properties=dict([(x.name, x) for x in properties]),
        qualifiers=dict([(x.name, x) for x in qualifiers]))

# <!ELEMENT QUALIFIER ((VALUE | VALUE.ARRAY)?)>
# <!ATTLIST QUALIFIER
#       %CIMName;
#       %CIMType;              #REQUIRED
#       %Propagated;
#       %QualifierFlavor;
#       xml:lang   NMTOKEN     #IMPLIED
# >

def parse_qualifier(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML QUALIFIER element and return CIMQualifier"""

    name = _get_required_attribute(node, 'NAME')
    cim_type = _get_required_attribute(node, 'TYPE')
    # TODO 2/16 KS: Why is propagated not used?
    propagated = _get_attribute(node, 'PROPAGATED')

    (next_event, next_node) = six.next(parser)

    if _is_end(next_event, next_node, 'QUALIFIER'):
        return CIMQualifier(name, None, type=cim_type)

    if _is_start(next_event, next_node, 'VALUE'):
        value = parse_value(parser, next_event, next_node)
    elif _is_start(next_event, next_node, 'VALUE.ARRAY'):
        #pylint: disable=redefined-variable-type
        # redefined from str to list.
        value = parse_value_array(parser, next_event, next_node)
    else:
        raise ParseError('Expecting (VALUE | VALUE.ARRAY)')

    result = CIMQualifier(name, tocimobj(cim_type, value))

    _get_end_event(parser, 'QUALIFIER')

    return result

# <!ELEMENT PROPERTY (QUALIFIER*, VALUE?)>
# <!ATTLIST PROPERTY
#       %CIMName;
#       %ClassOrigin;
#       %Propagated;
#       %CIMType;              #REQUIRED
#       xml:lang   NMTOKEN     #IMPLIED
# >

def parse_property(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML PROPERTY Element and return CIMProperty"""

    name = _get_required_attribute(node, 'NAME')
    cim_type = _get_required_attribute(node, 'TYPE')

    class_origin = _get_attribute(node, 'CLASSORIGIN')
    propagated = _get_attribute(node, 'PROPAGATED')

    qualifiers = []
    value = None

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'QUALIFIER'):

        qualifiers.append(parse_qualifier(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_start(next_event, next_node, 'VALUE'):
                break

            if _is_start(next_event, next_node, 'QUALIFIER'):
                qualifiers.append(
                    parse_qualifier(parser, next_event, next_node))
            else:
                raise ParseError('Expecting QUALIFIER')

    if _is_start(next_event, next_node, 'VALUE'):

        value = parse_value(parser, next_event, next_node)
        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'PROPERTY'):
        raise ParseError('Expecting end PROPERTY')

    return CIMProperty(
        name,
        tocimobj(cim_type, value),
        type=cim_type,
        class_origin=class_origin,
        propagated=propagated,
        qualifiers=dict([(x.name, x) for x in qualifiers]))

# <!ELEMENT PROPERTY.ARRAY (QUALIFIER*, VALUE.ARRAY?)>
# <!ATTLIST PROPERTY.ARRAY
#       %CIMName;
#       %CIMType;              #REQUIRED
#       %ArraySize;
#       %ClassOrigin;
#       %Propagated;
#       xml:lang   NMTOKEN     #IMPLIED
# >

def parse_property_array(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML PROPERTY.ARRAY element and return CIMProperty"""

    name = _get_required_attribute(node, 'NAME')
    cim_type = _get_required_attribute(node, 'TYPE')

    # TODO 2/16 KS: array_size here is unused. It is valid attribute.
    array_size = _get_attribute(node, 'ARRAYSIZE')
    class_origin = _get_attribute(node, 'CLASSORIGIN')
    propagated = _get_attribute(node, 'PROPAGATED')

    # TODO 1/16 KS: The qualifier processing could be common.
    qualifiers = []
    value = None

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'QUALIFIER'):

        qualifiers.append(parse_qualifier(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_start(next_event, next_node, 'VALUE.ARRAY'):
                break

            if _is_start(next_event, next_node, 'QUALIFIER'):
                qualifiers.append(
                    parse_qualifier(parser, next_event, next_node))
            else:
                raise ParseError('Expecting QUALIFIER')

    if _is_start(next_event, next_node, 'VALUE.ARRAY'):

        value = parse_value_array(parser, next_event, next_node)
        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'PROPERTY.ARRAY'):
        raise ParseError('Expecting end PROPERTY.ARRAY')

    return CIMProperty(
        name,
        tocimobj(type, value),
        type=cim_type,
        class_origin=class_origin,
        propagated=propagated,
        is_array=True,
        qualifiers=dict([(x.name, x) for x in qualifiers]))

# <!ELEMENT PROPERTY.REFERENCE (QUALIFIER*, (VALUE.REFERENCE)?)>
# <!ATTLIST PROPERTY.REFERENCE
#       %CIMName;
#       %ReferenceClass;
#       %ClassOrigin;
#       %Propagated;
# >

def parse_property_reference(parser, event, node): #pylint: disable=unused-argument
    """Parse CIM/XML PROPERTY.REFERENCE ELEMENT and return
       CIMProperty
    """

    name = _get_required_attribute(node, 'NAME')

    class_origin = _get_attribute(node, 'CLASSORIGIN')
    propagated = _get_attribute(node, 'PROPAGATED')

    qualifiers = []
    value = None

    (next_event, next_node) = six.next(parser)

    if _is_start(next_event, next_node, 'QUALIFIER'):

        qualifiers.append(parse_qualifier(parser, next_event, next_node))

        while 1:

            (next_event, next_node) = six.next(parser)

            if _is_start(next_event, next_node, 'VALUE.REFERENCE'):
                break

            if _is_start(next_event, next_node, 'QUALIFIER'):
                qualifiers.append(
                    parse_qualifier(parser, next_event, next_node))

            else:
                raise ParseError('Expecting QUALIFIER')

    if _is_start(next_event, next_node, 'VALUE.REFERENCE'):

        value = parse_value_reference(parser, next_event, next_node)
        (next_event, next_node) = six.next(parser)

    if not _is_end(next_event, next_node, 'PROPERTY.REFERENCE'):
        raise ParseError('Expecting end PROPERTY.REFERENCE')

    return CIMProperty(
        name,
        value,
        class_origin=class_origin,
        propagated=propagated,
        type='reference',
        qualifiers=dict([(x.name, x) for x in qualifiers]))

# <!ELEMENT METHOD (QUALIFIER*, (PARAMETER | PARAMETER.REFERENCE |
#                   PARAMETER.ARRAY | PARAMETER.REFARRAY)*)>
# <!ATTLIST METHOD
#       %CIMName;
#       %CIMType;              #IMPLIED
#       %ClassOrigin;
#       %Propagated;>

# <!ELEMENT PARAMETER (QUALIFIER*)>
# <!ATTLIST PARAMETER
#       %CIMName;
#       %CIMType;              #REQUIRED>

# <!ELEMENT PARAMETER.REFERENCE (QUALIFIER*)>
# <!ATTLIST PARAMETER.REFERENCE
#       %CIMName;
#       %ReferenceClass;
# >

# <!ELEMENT PARAMETER.ARRAY (QUALIFIER*)>
# <!ATTLIST PARAMETER.ARRAY
#       %CIMName;
#       %CIMType;              #REQUIRED
#       %ArraySize;>

# <!ELEMENT PARAMETER.REFARRAY (QUALIFIER*)>
# <!ATTLIST PARAMETER.REFARRAY
#       %CIMName;
#       %ReferenceClass;
#       %ArraySize;
# >

# <!ELEMENT TABLECELL.DECLARATION EMPTY>
# <!ATTLIST TABLECELL.DECLARATION
#       %CIMName;
#       %CIMType;                               #REQUIRED
#       ISARRAY         (true|false) "false"
#       %ArraySize;
#       CELLPOS         CDATA                   #REQUIRED
#       SORTPOS         CDATA                   #IMPLIED
#       SORTDIR         (ASC|DESC)              #IMPLIED
# >

# <!ELEMENT TABLECELL.REFERENCE EMPTY>
# <!ATTLIST TABLECELL.REFERENCE
#       %CIMName;
#       %ReferenceClass;
#       ISARRAY         (true|false) "false"
#       %ArraySize;
#       CELLPOS         CDATA                   #REQUIRED
#       SORTPOS         CDATA                   #IMPLIED
#       SORTDIR         (ASC|DESC)              #IMPLIED
# >

# <!ELEMENT TABLEROW.DECLARATION (TABLECELL.DECLARATION | TABLECELL.REFERENCE)*>

# <!ELEMENT TABLE (TABLEROW.DECLARATION,(TABLEROW)*)>

# <!ELEMENT TABLEROW (VALUE | VALUE.ARRAY | VALUE.REFERENCE | VALUE.REFARRAY |
#                     VALUE.NULL)*>

# <!-- ************************************************** -->
# <!-- Message elements                                   -->
# <!-- ************************************************** -->

# <!ELEMENT MESSAGE (SIMPLEREQ | MULTIREQ | SIMPLERSP | MULTIRSP |
#                    SIMPLEEXPREQ | MULTIEXPREQ | SIMPLEEXPRSP | MULTIEXPRSP)>
# <!ATTLIST MESSAGE
#       ID              CDATA #REQUIRED
#       PROTOCOLVERSION CDATA #REQUIRED
# >

# <!ELEMENT MULTIREQ (SIMPLEREQ, SIMPLEREQ+)>

# <!ELEMENT MULTIEXPREQ (SIMPLEEXPREQ, SIMPLEEXPREQ+)>

# <!ELEMENT SIMPLEREQ (IMETHODCALL | METHODCALL)>

# <!ELEMENT SIMPLEEXPREQ (EXPMETHODCALL)>

# <!ELEMENT IMETHODCALL (LOCALNAMESPACEPATH, IPARAMVALUE*,
#                        RESPONSEDESTINATION?)>
# <!ATTLIST IMETHODCALL
#       %CIMName;
# >

# <!ELEMENT METHODCALL ((LOCALINSTANCEPATH | LOCALCLASSPATH), PARAMVALUE*,
#                       RESPONSEDESTINATION?)>
# <!ATTLIST METHODCALL
#       %CIMName;
# >

# <!ELEMENT EXPMETHODCALL (EXPPARAMVALUE*)>
# <!ATTLIST EXPMETHODCALL
#       %CIMName;
# >

# <!ELEMENT PARAMVALUE (VALUE | VALUE.REFERENCE | VALUE.ARRAY |
#                       VALUE.REFARRAY)?>
# <!ATTLIST PARAMVALUE
#       %CIMName;
#       %ParamType;  #IMPLIED
# >

# <!ELEMENT IPARAMVALUE (VALUE | VALUE.ARRAY | VALUE.REFERENCE | INSTANCENAME |
#                        CLASSNAME | QUALIFIER.DECLARATION | CLASS | INSTANCE |
#                        VALUE.NAMEDINSTANCE)?>
# <!ATTLIST IPARAMVALUE
#       %CIMName;
# >

# <!ELEMENT EXPPARAMVALUE (INSTANCE? | VALUE? | METHODRESPONSE? |
#                          IMETHODRESPONSE?)>
# <!ATTLIST EXPPARAMVALUE
#       %CIMName;
# >

# <!ELEMENT MULTIRSP (SIMPLERSP, SIMPLERSP+)>

# <!ELEMENT MULTIEXPRSP (SIMPLEEXPRSP, SIMPLEEXPRSP+)>

# <!ELEMENT SIMPLERSP (METHODRESPONSE | IMETHODRESPONSE | SIMPLEREQACK)>

# <!ELEMENT SIMPLEEXPRSP (EXPMETHODRESPONSE)>

# <!ELEMENT METHODRESPONSE (ERROR | (RETURNVALUE?, PARAMVALUE*))>
# <!ATTLIST METHODRESPONSE
#       %CIMName;
# >

# <!ELEMENT EXPMETHODRESPONSE (ERROR | IRETURNVALUE?)>
# <!ATTLIST EXPMETHODRESPONSE
#       %CIMName;
# >

# <!ELEMENT IMETHODRESPONSE (ERROR | IRETURNVALUE?)>
# <!ATTLIST IMETHODRESPONSE
#       %CIMName;
# >

# <!ELEMENT ERROR (INSTANCE*)>
# <!ATTLIST ERROR
#       CODE        CDATA #REQUIRED
#       DESCRIPTION CDATA #IMPLIED
# >

# <!ELEMENT RETURNVALUE (VALUE | VALUE.REFERENCE)>
# <!ATTLIST RETURNVALUE
#       %ParamType;       #IMPLIED
# >

# <!ELEMENT IRETURNVALUE (CLASSNAME* | INSTANCENAME* | VALUE* |
#                         VALUE.OBJECTWITHPATH* | VALUE.OBJECTWITHLOCALPATH* |
#                         VALUE.OBJECT* | OBJECTPATH* | QUALIFIER.DECLARATION* |
#                         VALUE.ARRAY? | VALUE.REFERENCE? | CLASS* | INSTANCE* |
#                         VALUE.NAMEDINSTANCE*)>

# <!ELEMENT RESPONSEDESTINATION (INSTANCE)>

# <!ELEMENT SIMPLEREQACK (ERROR?)>
# <!ATTLIST SIMPLEREQACK
#       INSTANCEID CDATA     #REQUIRED
# >

def make_parser(stream_or_string):
    """Create a xml.dom.pulldom parser."""

    if isinstance(stream_or_string, six.string_types):

        # XXX: the pulldom.parseString() function doesn't seem to
        # like operating on unicode strings!

        return pulldom.parseString(str(stream_or_string))

    else:

        return pulldom.parse(stream_or_string)

def parse_any(stream_or_string):
    """Parse any XML string or stream. This function fabricates
       the names of the parser functions by prepending parse_ to
       the node name and then calling that function.
    """

    parser = make_parser(stream_or_string)

    (event, node) = six.next(parser)

    if event != pulldom.START_DOCUMENT:
        raise ParseError('Expecting document start')

    (event, node) = six.next(parser)

    if event != pulldom.START_ELEMENT:
        raise ParseError('Expecting element start')

    fn_name = 'parse_%s' % node.tagName.lower().replace('.', '_')
    fn = globals().get(fn_name)
    if fn is None:
        raise ParseError('No parser for element %s' % node.tagName)

    return fn(parser, event, node)

# Test harness

if __name__ == '__main__':
    print(parse_any(sys.stdin))



#!/usr/bin/python

'''
Script that automatically builds a table of contents (TOC) for an IPython
notebook.
Returns string to paste into notebook.
'''

import sys
import json


def main(ifilepath):

    with open(ifilepath, 'r') as infile:
        notebook = json.loads(infile.read())

    cells = notebook['cells']
    titles = []

    for cell in cells:
        if cell['cell_type'] == 'markdown':

            # /!\ may be multiple headers in the markdown cell
            cell_titles = filter(lambda line: line.startswith('#'),
                                 cell['source'])
            cell_titles = map(lambda title: title.strip('\n'), cell_titles)

            for md_title in cell_titles:
                # store header importance to show it in the toc
                level = md_title.count('#') - 1
                title = md_title.lstrip('#').strip()
                titles.append((level, title))

    levels, titles = zip(*titles)
    urls = [title_.replace(' ', '-') for title_ in titles]

    toc = ['- [%s](#%s)\n' % (title, url) for title, url in zip(titles, urls)]
    toc = ['\t%s' % tt if lvl > 1 else tt for tt, lvl in zip(toc, levels)]
    toc = [tt for tt, lvl in zip(toc, levels) if lvl <= 5]

    toc = ''.join(toc)
    return toc


if __name__ == '__main__':

    ifilepath = sys.argv[1]
    toc = main(ifilepath)
print(toc)



#!/usr/bin/python
#
# Simple indication receiver using Twisted Python.  HTTP post requests
# are listened for on port 5988 and port 5899 using SSL.
#
# Requires Twisted Python and
#

import sys
from twisted.internet import reactor
from twisted.web import server, resource
import pywbem
import threading

from twisted.internet import ssl, reactor
from twisted.python import log

from time import sleep

class CIMListener(resource.Resource):
    """ CIM Listener
    """

    isLeaf = 1

    class ServerContextFactory(object):
        def __init__(self, cert, key):
            self.cert = cert
            self.key = key

        def getContext(self):
            """Create an SSL context with a dodgy certificate."""

            from OpenSSL import SSL
            ctx = SSL.Context(SSL.SSLv23_METHOD)
            ctx.use_certificate_file(self.cert)
            ctx.use_privatekey_file(self.key)
            return ctx

    def __init__(self, callback,
            http_port=5988, https_port=5989,
            ssl_key=None, ssl_cert=None):
        self.callback = callback
        self.http_port = http_port
        self.https_port = https_port
        self.ssl_key = ssl_key
        self.ssl_cert = ssl_cert

        site = server.Site(self)

        if self.http_port and self.http_port > 0:
            reactor.listenTCP(self.http_port, site)
        if self.https_port and self.https_port > 0:
            reactor.listenSSL(self.https_port, site,
                    self.ServerContextFactory(cert=ssl_cert, key=ssl_key))

    def start(self):
        ''' doesn't work'''
        thread = threading.Thread(target=reactor.run)
        thread.start()

    def stop(self):
        reactor.stop()

    def run(self):
        reactor.run()

    def render_POST(self, request):
        tt = pywbem.parse_cim(pywbem.xml_to_tupletree(request.content.read()))
        insts = [x[1] for x in tt[2][2][0][2][2]]
        for inst in insts:
            self.callback(inst)
        return ''


#log.startLogging(sys.stdout)

if __name__ == '__main__':
    def cb(inst):
        print('%s %s' % (inst['IndicationTime'], inst['Description']))
    cl = CIMListener(cb, https_port=None)
    cl.run()





"""
    Pywbem wbemcli scriptlet that tests wbemcli shortcut methods.  The
    goal of this scriptlet is to test the parameters for all of the
    shortcut methods in wbemcli to assure that the names are correct and
    that they execute.  This does not guarantee that the results are always
    correct.

    This scriptlet requires a running server.

    It also depends heavily on the existence of the PyWBEM special
    classes and instances in the server.

    This scriptlet executes an assert to test results but does NOT use
    the python unittest library.

    NOTE: TODO does not test the create/delete functions for instances,
           classes or QualifierDeclarations.
"""
from wbemcli import CONN, gc, ecn, ei, iei, oei, piwp, ein, ieip, oeip, pip, \
    a, iai, oai, an, iaip, oaip, iri, ori, rn, irip, orip, ec, gq, eq

# test objects
# live class against the server. may not always return same number of elements
TEST_CLASS1 = 'CIM_ManagedElement'
# class that returns a defined set of elements
TEST_CLASS2 = 'PyWBEM_Person'
TEST_CLASS2_PROPERTIES = ['Name', 'CreationClassName']

# pywbem test classes. Can be used to test associations because association
# definitions are specific.
PYWBEM_PERSON_CLASS = "PyWBEM_Person"
PYWBEM_PERSON_PROPERTIES = ['InstanceId', 'CreationClassName']
PYWBEM_PERSONCOLLECTION = "PyWBEM_PersonCollection"
PYWBEM_MEMBEROFPERSONCOLLECTION = "PyWBEM_MemberOfPersonCollection"
PYWBEM_SOURCE_ROLE = "member"
PYWBEM_TARGET_ROLE = "collection"
PERSON_COUNT = 3


def test_enum_insts(cn, ns=None, lo=None, iq=None, ico=None, pl=None, fl=None,
                    fq=None, ot=None, coe=None, moc=1, mop=10, exp_count=None):
    """
    Function to execute all 3 types of enumerate instances and compare
    results. Executes EnumerateInstances, Open/Pull instances with path and
    IterEnumerateInstances and compares to see if the same number of instances
    received.
    """
    insts_ei = ei(cn, ns=ns, lo=lo, iq=iq, ico=ico, pl=pl)
    assert len(insts_ei) > 0
    if exp_count:
        assert exp_count == len(insts_ei)

    if pl is not None:
        # test for existence of first property
        for prop in pl:
            # test that all properties in list are returned
            pls = [inst[prop] for inst in insts_ei]
            assert len(pls)
        for inst in insts_ei:
            assert len(inst.properties) == len(pl)

    insts_iei = [inst for inst in iei(cn, ns=ns, lo=lo, iq=iq, ico=ico,
                                      pl=pl, fl=fl, fq=fq, ot=ot, coe=coe,
                                      moc=moc)]
    assert len(insts_ei) == len(insts_iei)
    result = oei(cn, ns=ns, lo=lo, iq=iq, ico=ico, pl=pl, fl=fl, fq=fq,
                 ot=ot, coe=coe, moc=moc)
    insts_pei = result.instances
    while not result.eos:
        result = piwp(result.context, moc=mop)
        insts_pei.extend(result.instances)
    assert len(insts_ei) == len(insts_pei)


def test_enum_instpaths(cn, ns=None, fl=None, fq=None, ot=None, coe=None,
                        moc=1, mop=10, exp_count=None):
    """
    Function to execute all 3 types of enumerate instances and compare
    results.
    """
    insts_ei = ein(cn, ns=ns)
    assert len(insts_ei) > 0
    if exp_count:
        exp_count = len(insts_ei)

    insts_iei = [inst for inst in ieip(cn, ns=ns, fl=fl, fq=fq, ot=ot,
                                       coe=coe, moc=moc)]
    assert len(insts_ei) == len(insts_iei)
    result = oeip(cn, ns=ns, fl=fl, fq=fq, ot=ot, coe=coe, moc=moc)
    insts_pei = result.paths
    while not result.eos:
        result = pip(result.context, moc=mop)
        insts_pei.extend(result.paths)
    assert len(insts_ei) == len(insts_pei)


def test_instassocs(cn, ac=None, rc=None, r=None, rr=None, iq=None, ico=None,
                    pl=None, fl=None, fq=None, ot=None, coe=None, moc=None,
                    mop=10, exp_count=None):  # pylint: disable=invalid-name
    """
    Function to execute all 3 types of enumerate instance paths and compare
    results.
    """
    # first get an instance to work with
    paths_ei = ein(cn)
    assert len(paths_ei) > 0
    tst_path = paths_ei[0]

    insts_a = a(tst_path, ac=ac, rc=rc, r=r, rr=rr, iq=iq, ico=ico,
                pl=pl)
    if not moc:
        moc = 1
    insts_iai = [inst for inst in iai(tst_path, ac=ac, rc=rc, r=r, rr=rr,
                                      iq=iq, ico=ico, pl=pl, fl=fl, fq=fq,
                                      ot=ot, coe=coe, moc=moc)]
    if exp_count:
        assert exp_count == len(insts_a)

    assert len(insts_a) == len(insts_iai)
    # get with pull operations
    result = oai(tst_path, ac=ac, rc=rc, r=r, rr=rr, iq=iq, ico=ico, pl=pl,
                 fl=fl, fq=fq, ot=ot, coe=coe, moc=moc)
    insts_pai = result.instances
    while not result.eos:
        result = piwp(result.context, moc=mop)
        insts_pai.extend(result.instances)
    assert len(insts_a) == len(insts_pai)


def test_instassocpaths(cn, ac=None, rc=None, r=None, rr=None,
                        fl=None, fq=None, ot=None, coe=None, moc=None, mop=10,
                        exp_count=None):  # pylint: disable=invalid-name
    """
    Function to execute all 3 types of enumerate instance paths  requests
    and compare results.
    """
    # first get an instance to work with
    paths_ei = ein(cn)
    assert len(paths_ei) > 0
    tst_path = paths_ei[0]

    paths_an = an(tst_path, ac=ac, rc=rc, r=r, rr=rr)
    if not moc:
        moc = 1
    paths_iai = [path for path in iaip(tst_path, ac=ac, rc=rc, r=r, rr=rr,
                                       fl=fl, fq=fq, ot=ot, coe=coe, moc=moc)]
    if exp_count:
        assert exp_count == len(paths_an)

    assert len(paths_an) == len(paths_iai)
    # get with pull operations
    result = oaip(tst_path, ac=ac, rc=rc, r=r, rr=rr, fl=fl,
                  fq=fq, ot=ot, coe=coe, moc=moc)
    paths_pai = result.paths
    while not result.eos:
        result = pip(result.context, moc=mop)
        paths_pai.extend(result.paths)
    assert len(paths_an) == len(paths_pai)


def test_instrefs(cn, rc=None, r=None, iq=None, ico=None,
                  pl=None, fl=None, fq=None, ot=None, coe=None, moc=None,
                  mop=10, exp_count=None):
    """
    Function to execute all 3 types of enumerate instance paths and compare
    results. This test function takes a class as input and get the first
    instance of that class for the reference call.
    """
    # first get an instance to work with
    paths_ei = ein(cn)
    assert len(paths_ei) > 0
    tst_path = paths_ei[0]

    # TODO using r here causes failure with non-callable object
    insts_r = CONN.References(tst_path, ResultClass=rc, Role=r,
                              IncludeQualifiers=iq, IncludeClassOrigin=ico,
                              PropertyList=pl)
    if not moc:
        moc = 1
    insts_iri = [inst for inst in iri(tst_path, rc=rc, r=r,
                                      iq=iq, ico=ico, pl=pl, fl=fl, fq=fq,
                                      ot=ot, coe=coe, moc=moc)]
    if exp_count:
        assert exp_count == len(insts_r)
    assert len(insts_r) == len(insts_iri)
    # get with pull operations
    result = ori(tst_path, rc=rc, r=r, iq=iq, ico=ico, pl=pl,
                 fl=fl, fq=fq, ot=ot, coe=coe, moc=moc)
    insts_pri = result.instances
    while not result.eos:
        result = piwp(result.context, moc=mop)
        insts_pri.extend(result.instances)
    assert len(insts_r) == len(insts_pri)


def test_instrefpaths(cn, rc=None, r=None,
                      fl=None, fq=None, ot=None, coe=None, moc=None, mop=10,
                      exp_count=None):  # pylint: disable=invalid-name
    """
    Function to execute all 3 types of enumerate instance paths and compare
    results.
    """
    # first get an instance to work with
    paths_ei = ein(cn)
    assert len(paths_ei) > 0
    tst_path = paths_ei[0]

    paths_rn = rn(tst_path, rc=rc, r=r)
    if not moc:
        moc = 1
    paths_irip = [path for path in irip(tst_path, rc=rc, r=r,
                                        fl=fl, fq=fq, ot=ot, coe=coe, moc=moc)]
    if exp_count:
        assert exp_count == len(paths_rn)

    assert len(paths_irip) == len(paths_irip)
    # get with pull operations
    result = orip(tst_path, rc=rc, r=r, fl=fl,
                  fq=fq, ot=ot, coe=coe, moc=moc)
    paths_pai = result.paths
    while not result.eos:
        result = pip(result.context, moc=mop)
        paths_pai.extend(result.paths)
    assert len(paths_rn) == len(paths_pai)


# test getclass
cl = gc(PYWBEM_PERSON_CLASS)  # pylint: disable=invalid-name
assert cl.classname == PYWBEM_PERSON_CLASS

# test enumerate classnames
clns = ecn()  # pylint: disable=invalid-name
assert 'CIM_ManagedElement' in clns

# test enumerate classes
cls = ec()  # pylint: disable=invalid-name
assert cls

# test get QualifierDeclarations
qd = gq('Abstract')  # pylint: disable=invalid-name
assert qd

# test enumerate qualifierdeclarations
qds = eq()  # pylint: disable=invalid-name
assert qds

# Tests executed against the common function to test enumerate instances
# This tests EnumerateInstances, Open/PullEnumerateInstances, and
# IterEnumerateInstances
test_enum_insts(TEST_CLASS2)
test_enum_insts(TEST_CLASS2, pl=TEST_CLASS2_PROPERTIES)
# this one returns error LocalOnly not supported???
# test_enum_insts(TEST_CLASS2, pl=TEST_CLASS2_PROPERTIES, lo=True)
test_enum_insts(TEST_CLASS2, pl=[TEST_CLASS2_PROPERTIES[0]])
test_enum_insts(TEST_CLASS2, iq=True)
test_enum_insts(TEST_CLASS2, iq=False)
test_enum_insts(TEST_CLASS2, ico=True)
test_enum_insts(TEST_CLASS2, ico=False)
# TODO not testing fq, fl, coe, ot, ns

# Test that the following returns the correct count
test_enum_insts(PYWBEM_PERSON_CLASS, exp_count=PERSON_COUNT)

# Test enumerate paths functions
# TODO not testing ns
test_enum_instpaths(TEST_CLASS2)

# Test association functions
# TODO not testing fq, fl, ot coe
test_instassocs(PYWBEM_PERSON_CLASS)
test_instassocs(PYWBEM_PERSON_CLASS, ot=30)
test_instassocs(PYWBEM_PERSON_CLASS, exp_count=1)
test_instassocs(PYWBEM_PERSON_CLASS, exp_count=1,
                ac=PYWBEM_MEMBEROFPERSONCOLLECTION,
                rc=PYWBEM_PERSONCOLLECTION,
                r=PYWBEM_SOURCE_ROLE,
                rr=PYWBEM_TARGET_ROLE)

test_instassocpaths(PYWBEM_PERSON_CLASS)
test_instassocpaths(PYWBEM_PERSON_CLASS, exp_count=1)
test_instassocpaths(PYWBEM_PERSON_CLASS, exp_count=1,
                    ac=PYWBEM_MEMBEROFPERSONCOLLECTION,
                    rc=PYWBEM_PERSONCOLLECTION,
                    r=PYWBEM_SOURCE_ROLE,
                    rr=PYWBEM_TARGET_ROLE)

# Test reference functions
test_instrefs(PYWBEM_PERSON_CLASS)
test_instrefs(PYWBEM_PERSON_CLASS, exp_count=1)
test_instrefs(PYWBEM_PERSON_CLASS, exp_count=1,
              rc=PYWBEM_MEMBEROFPERSONCOLLECTION,
              r=PYWBEM_SOURCE_ROLE)

test_instrefpaths(PYWBEM_PERSON_CLASS)
test_instrefpaths(PYWBEM_PERSON_CLASS, exp_count=1)
test_instrefpaths(PYWBEM_PERSON_CLASS, exp_count=1,
                  rc=PYWBEM_MEMBEROFPERSONCOLLECTION,
                  r=PYWBEM_SOURCE_ROLE)

print('Test successful. Quitting')
quit()



#!/usr/bin/python
#
# Simple indication receiver using Twisted Python.  HTTP post requests
# are listened for on port 5988 and port 5899 using SSL.
#
# Requires Twisted Python and
#

import sys
import optparse
import pywbem
from twisted.internet import reactor
from twisted.web import server, resource

global conn
conn=None

class WBEMConn:
    _shared_state = {}
    conn = None

    def __init__(self, options=None):
        # Borgness
        self.__dict__ = WBEMConn._shared_state
        self.conn = pywbem.SFCBUDSConnection()
        '''
        if options:
            proto = 'http'
            if options.secure:
                proto = 'https'
            url = '%s://%s' % (proto, options.host)
            self.conn = pywbem.WBEMConnection(
                    url,
                    (options.user, options.password),
                    default_namespace = options.namespace)
        '''
        global conn
        conn = self.conn



class CIMOM(resource.Resource):

    isLeaf = 1

    def render_POST(self, request):

        for line in request.content.readlines():
            print(line)

        return ''

from OpenSSL import SSL

class ServerContextFactory:
    def getContext(self):
        """Create an SSL context with a dodgy certificate."""
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.use_certificate_file('server.pem')
        ctx.use_privatekey_file('server.pem')
        return ctx

from twisted.internet import ssl, reactor
from twisted.python import log
from socket import getfqdn
import time

def _createFilter(query,
                  ns,
                  querylang='WQL',
                  src_ns='root/cimv2',
                  in_name=None):
    name = in_name or 'cimfilter%s'%time.time()
    filterinst=pywbem.CIMInstance('CIM_IndicationFilter')
    filterinst['CreationClassName']='CIM_IndicationFilter'
    filterinst['SystemCreationClassName']='CIM_ComputerSystem'
    filterinst['SystemName']=getfqdn()
    filterinst['Name']=name
    filterinst['Query']=query
    filterinst['QueryLanguage']=querylang
    filterinst['SourceNamespace']=src_ns
    cop = pywbem.CIMInstanceName('CIM_IndicationFilter')
    cop.keybindings = { 'CreationClassName':'CIM_IndicationFilter',
                        'SystemClassName':'CIM_ComputerSystem',
                        'SystemName':getfqdn(),
                        'Name':name }
    cop.namespace=ns
    filterinst.path = cop
    filtercop = conn.CreateInstance(filterinst)
    return filtercop

def _createDest(destination,
                ns,
                in_name=None):
    name = in_name or 'cimlistener%s'%time.time()
    destinst=pywbem.CIMInstance('CIM_ListenerDestinationCIMXML')
    destinst['CreationClassName']='CIM_ListenerDestinationCIMXML'
    destinst['SystemCreationClassName']='CIM_ComputerSystem'
    destinst['SystemName']=getfqdn()
    print("destname=%s" % name)
    destinst['Name']=name
    destinst['Destination']=destination
    cop = pywbem.CIMInstanceName('CIM_ListenerDestinationCIMXML')
    cop.keybindings = { 'CreationClassName':'CIM_ListenerDestinationCIMXML',
                        'SystemClassName':'CIM_ComputerSystem',
                        'SystemName':getfqdn(),
                        'Name':name }
    cop.namespace=ns
    destinst.path = cop
    destcop = conn.CreateInstance(destinst)
    return destcop

def _createSubscription(ns,
                        handler,
                        indfilter):
    subinst=pywbem.CIMInstance('CIM_IndicationSubscription')
    subinst['Filter']=indfilter
    subinst['Handler']=indhandler
    cop = pywbem.CIMInstanceName('CIM_IndicationSubscription')
    cop.keybindings = { 'Filter':indfilter,
                        'Handler':indhandler }
    cop.namespace=ns
    subinst.path = cop
    subcop = conn.CreateInstance(subinst)
    return subcop


if __name__ == '__main__':
    global conn
    parser = optparse.OptionParser()
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    parser.add_option('-s', '--UDS', help="Use the SFCBUDSConnection to the cimom", default=False )
    parser.add_option('-u', '--url', default='https://localhost',
            help='Specify the url of the CIMOM (default=https://localhost)')
    parser.add_option('-n', '--namespace', default='root/interop',
            help='Specify the namespace the test runs against (default=root/interop)')
    parser.add_option('', '--user', default='pegasus',
            help='Specify the user name used when connection to the CIMOM (default=pegasus)')
    parser.add_option('', '--password', default='',
            help='Specify the password for the user (default=<empty>)')
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')

    parser.add_option('-q', '--query', help='Query string for Filter')
    parser.add_option('-g', '--qlang', help='Query Language (default=WQL)', default="WQL")
    parser.add_option('-d', '--dest', help='Destination for the CIM_ListenerDestination')

    parser.add_option('-p', '--provider', help='Name of provider to setup listener for')
    options, arguments = parser.parse_args()

    conn = WBEMConn().conn

    indhandler=None
    indfilter=None
    indsub=None
    try:
        indhandler = _createDest(options.dest, options.namespace)
        indfilter = _createFilter(options.query, options.namespace, querylang=options.qlang)
        indsub = _createSubscription(options.namespace, indhandler, indfilter)

        log.startLogging(sys.stdout)

        site = server.Site(CIMOM())

        reactor.listenTCP(5998, site)
        reactor.listenSSL(5999, site, ServerContextFactory())
        reactor.run()
    finally:
        if indsub:
            conn.DeleteInstance(indsub)
        if indfilter:
            conn.DeleteInstance(indfilter)
        if indhandler:
            conn.DeleteInstance(indhandler)

